/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "analogWrite.h"
#include "esp32-hal-ledc.h"
#include "sim_common_definitions.h"

typedef struct analog_write_channel
{
  int pin;
  unsigned char configured;
} analog_write_channel_t;

static volatile analog_write_channel_t g_analog_write_channels[16] = {
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0},
    {-1, 0}};

int analogWriteChannel(int pin)
{
    for(int i=0; i<16; i++)
    {
        if(g_analog_write_channels[i].pin == pin)
            return i;
    }
    for(int i=0; i<16; i++)
    {
        if(g_analog_write_channels[i].pin == -1)
        {
            g_analog_write_channels[i].pin = pin;
            return i;
        }
    }
    return -1;
}

void analogWriteResolution(unsigned int res)
{
    //this is kept for compatibility with Arduino lib. Resolution is set to CASP default in analogWrite function
}

void analogResetConfiguration(int pin)
{
    int channel = analogWriteChannel(pin);
    if(channel > -1)
        g_analog_write_channels[channel].configured = 0;
}

void analogWriteConfigure(int pin, int channel, unsigned int resolution, float freq)
{
    if(g_analog_write_channels[channel].configured == 0)
    {
        ledcSetup(channel+1, freq, resolution);
        ledcAttachPin(pin, channel+1);
        g_analog_write_channels[channel].configured = 1;
    }
}

void analogWrite(int pin, unsigned int value)
{
    int channel = analogWriteChannel(pin);
    if(channel < 0)
        return;
    if(g_analog_write_channels[channel].configured == 0)
        analogWriteConfigure(pin, channel);
    ledcWrite(channel+1, value);
}

void analogReadResolutionPin(unsigned char res, unsigned char pin)
{

}

void analogWriteResolutionPin(unsigned char res, unsigned char pin)
{

}

void pinCore(unsigned char pin, unsigned char mode, unsigned char core_idx)
{
    //in esp32, one pwm channel is alloted to 2 pins. so be carefull when using servos. as such
    //this is developed to initialize normal pwm channels before servo pwms are initialized
//    if(mode == IO_PIN_PWM)
//    {
//        int channel = analogWriteChannel(pin);
//        if(channel > -1)
//        {
//            if(g_analog_write_channels[channel].configured == 0)
//                analogWriteConfigure(pin, channel);
//        }
//    }
}
